#include <windows.h>
#include "internal.h"

#define MAGIC_NUMBER (0xdeadbeef)

bool PutHintingData(unsigned char *video, unsigned int hint)
{
	unsigned char *p;
	unsigned int i, magic_number = MAGIC_NUMBER;
	bool error = false;

	p = video;
	for (i = 0; i < 32; i++)
	{
		*p &= ~1; 
		*p++ |= ((magic_number & (1 << i)) >> i);
	}
	for (i = 0; i < 32; i++)
	{
		*p &= ~1;
		*p++ |= ((hint & (1 << i)) >> i);
	}
	return error;
}

bool GetHintingData(unsigned char *video, unsigned int *hint)
{
	unsigned char *p;
	unsigned int i, magic_number = 0;
	bool error = false;

	p = video;
	for (i = 0; i < 32; i++)
	{
		magic_number |= ((*p++ & 1) << i);
	}
	if (magic_number != MAGIC_NUMBER)
	{
		error = true;
	}
	else
	{
		*hint = 0;
		for (i = 0; i < 32; i++)
		{
			*hint |= ((*p++ & 1) << i);
		}
	}
	return error;
}

#ifdef MAKEWRITABLEHACK

bool MakeWritable2(PVideoFrame* pvf, IScriptEnvironment* env, VideoInfo vi)
{
    const PVideoFrame& vf = *pvf;
    int x, y, row_size, height, spitch, dpitch;
    unsigned char *srcp, *dstp;

	if (vf->IsWritable())
	{
//		return false;
	}

	// Allocate a new frame (using NewVideoFrame) and
	// copy the data into it.  Then modify the passed PVideoFrame
	// to point to the new buffer.
	PVideoFrame dst = env->NewVideoFrame(vi);
	if (vi.IsYV12())
	{
		row_size = vf->GetRowSize(PLANAR_Y);
		height = vf->GetHeight(PLANAR_Y);
		spitch = vf->GetPitch(PLANAR_Y);
		srcp = (unsigned char *) vf->GetReadPtr(PLANAR_Y);
		dpitch = dst->GetPitch(PLANAR_Y);
		dstp = (unsigned char *) dst->GetWritePtr(PLANAR_Y);
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < row_size; x++)
			{
				dstp[x] = srcp[x];
			}
			srcp += spitch;
			dstp += dpitch;
		}

		row_size = vf->GetRowSize(PLANAR_U);
		height = vf->GetHeight(PLANAR_U);
		spitch = vf->GetPitch(PLANAR_U);
		srcp = (unsigned char *) vf->GetReadPtr(PLANAR_U);
		dpitch = dst->GetPitch(PLANAR_U);
		dstp = (unsigned char *) dst->GetWritePtr(PLANAR_U);
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < row_size; x++)
			{
				dstp[x] = srcp[x];
			}
			srcp += spitch;
			dstp += dpitch;
		}

		row_size = vf->GetRowSize(PLANAR_V);
		height = vf->GetHeight(PLANAR_V);
		spitch = vf->GetPitch(PLANAR_V);
		srcp = (unsigned char *) vf->GetReadPtr(PLANAR_V);
		dpitch = dst->GetPitch(PLANAR_V);
		dstp = (unsigned char *) dst->GetWritePtr(PLANAR_V);
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < row_size; x++)
			{
				dstp[x] = srcp[x];
			}
			srcp += spitch;
			dstp += dpitch;
		}
	}
	else if (vi.IsYUY2())
	{
		row_size = vf->GetRowSize();
		height = vf->GetHeight();
		spitch = vf->GetPitch();
		srcp = (unsigned char *) vf->GetReadPtr();
		dpitch = dst->GetPitch();
		dstp = (unsigned char *) dst->GetWritePtr();
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < row_size; x++)
			{
				dstp[x] = srcp[x];
			}
			srcp += spitch;
			dstp += dpitch;
		}
	}

    *pvf = dst;
    return true;
}

#endif