#include <stdio.h>
#include <windows.h>
#include "internal.h"
#include "version.h"
#include "utilities.h"
#include "info.h"
#include "repal.h"

extern void DrawString(PVideoFrame &dst, int x, int y, const char *s);

void rePal::DrawShow(PVideoFrame &src, int useframe, int dropframe,
						 int inframe, int calc_dropframe, IScriptEnvironment* env)
{
	char buf[80];

	if (show == true)
	{
#ifdef MAKEWRITABLEHACK
		MakeWritable2(&src, env, vi);
#else
		env->MakeWritable(&src);
#endif
		sprintf(buf, "rePal %s", VERSION);
		DrawString(src, 0, 0, buf);
		sprintf(buf, "Copyright 2003 Bach");
		DrawString(src, 0, 1, buf);
		DrawString(src, 0, 1, buf);
				sprintf(buf,"in frm %d, use frm %d", inframe, useframe);
				DrawString(src, 0, 3, buf);
				sprintf(buf,"anchor frm %d (type= %d)", dropframe, dropframe%12);
				DrawString(src, 0, 4, buf);	
				sprintf(buf,"Should be frm %d (type= %d)", calc_dropframe, calc_dropframe%12);
				DrawString(src, 0, 5, buf);	
				sprintf(buf,"difference %d", (dropframe-calc_dropframe)%12);
				DrawString(src, 0, 6, buf);	
	}
}

PVideoFrame __stdcall rePal::GetFrame(int inframe, IScriptEnvironment* env)
{
	if (vi.IsYV12()) return(GetFrameYV12(inframe, env));
	else if (vi.IsYUY2()) return(GetFrameYUY2(inframe, env));
	env->ThrowError("rePal: YUY2 or YV12 data only");
	return 0;	
}

PVideoFrame __stdcall rePal::GetFrameYV12(int inframe, IScriptEnvironment* env)
{
	int dropframe, useframe, calc_dropframe;
	PVideoFrame src, next, dst;
	char buf[255];

	/* 60fps -> 25fps, 7 in 12 decimation*/
	/* Determine the correct set of frames to use and get them */
	useframe = inframe + inframe*7/5 ;
	FindAnchor((useframe/12)*12, &dropframe, &calc_dropframe, env);
	switch(dropframe%12)
		{
		case 1:
			useframe++;
			break;
		case 2:
			useframe+=2;
			break;
		case 3:
			useframe++;
			if(dropframe==useframe+2 || dropframe==useframe-5) useframe--;
			break;
		case 4:
			useframe+=2;
			if(dropframe==useframe+2 || dropframe==useframe-5) useframe--;
			break;		
		case 5:
			if(dropframe==useframe+1) useframe++;
			break;
		case 6:
			useframe++;
			if(dropframe==useframe+1) useframe++;
			break;
		case 7:
			useframe+=2;
			if(dropframe==useframe+1) useframe++;
			break;
		case 8:
			useframe++;
			if(dropframe==useframe+7) useframe--;
			break;
        case 9:
			useframe+=2;
			if(dropframe==useframe+7) useframe--;
			break;
		case 10:
			if(dropframe==useframe+6 || dropframe==useframe+1) useframe++;
			break;
		case 11:
			useframe++;
			if(dropframe==useframe+6 || dropframe==useframe+1) useframe++;
			break;
		default:
			break;
		}
	    
	   	GETFRAME(useframe, src);

		if (show == true)
		{
#ifdef MAKEWRITABLEHACK
			MakeWritable2(&src, env, vi);
#else
			env->MakeWritable(&src);
#endif
				sprintf(buf, "rePal %s", VERSION);
				DrawString(src, 0, 0, buf);
				sprintf(buf, "Copyright 2003 Bach");
				DrawString(src, 0, 1, buf);
				sprintf(buf,"in frm %d, use frm %d", inframe, useframe);
				DrawString(src, 0, 3, buf);
				sprintf(buf,"anchor frm %d (type= %d)", dropframe, dropframe%12);
				DrawString(src, 0, 4, buf);	
				sprintf(buf,"Should be frm %d (type= %d)", calc_dropframe, calc_dropframe%12);
				DrawString(src, 0, 5, buf);	
				sprintf(buf,"difference %d", (dropframe-calc_dropframe)%12);
				DrawString(src, 0, 6, buf);	
		}
	return src;
}

void __stdcall rePal::FindAnchor(int frame, int *forced_chosen, int *chosen, IScriptEnvironment* env)

{
	int f;
	PVideoFrame store[13];
	const unsigned char *storepY[13];
	const unsigned char *storepU[13];
	const unsigned char *storepV[13];
	const unsigned char *prevY, *prevU, *prevV, *currY, *currU, *currV;
	int x, y, lowest_index, div;
	unsigned int count[12], lowest;
	
	/* Only recalculate differences when a new set is needed. */
	if (frame == last_request)
	{
		*forced_chosen = last_result;
		*chosen = last_chosen;
		return;
	}
	last_request = frame;
	if (firsttime == true || frame == 0)
	{
		firsttime=false;
		last_result=1;
	}
	mod_=last_result%12;

	/* Get 13 frames starting at the one before the asked-for one. */
	for (f = 0; f <= 12; f++)
	{
		GETFRAME(frame + f - 1, store[f]);
		storepY[f] = store[f]->GetReadPtr(PLANAR_Y);
		hints_invalid = GetHintingData((unsigned char *) storepY[f], &hints[f]);
		if (quality == 1 || quality == 3)
		{
			storepU[f] = store[f]->GetReadPtr(PLANAR_U);
			storepV[f] = store[f]->GetReadPtr(PLANAR_V);
		}
	}

    pitchY = store[0]->GetPitch(PLANAR_Y);
    row_sizeY = store[0]->GetRowSize(PLANAR_Y);
    heightY = store[0]->GetHeight(PLANAR_Y);
	if (quality == 1 || quality == 3)
	{
		pitchUV = store[0]->GetPitch(PLANAR_V);
		row_sizeUV = store[0]->GetRowSize(PLANAR_V);
		heightUV = store[0]->GetHeight(PLANAR_V);
	}
	switch (quality)
	{
	case 0: // subsample, luma only
		div = (heightY * row_sizeY / 4) * 219;
		break;
	case 1: // subsample, luma and chroma
		div = (heightY * row_sizeY / 4) * 219 + (2 * (heightUV * row_sizeUV / 4)) * 224;
		break;
	case 2: // fully sample, luma only
		div = (heightY * row_sizeY) * 219;
		break;
	case 3: // fully sample, luma and chroma
		div = (heightY * row_sizeY) * 219 + (2 * heightUV * row_sizeUV) * 224;
		break;
	}

	/* Compare each frame to its predecessor. */
	for (f = 1; f <= 12; f++)
	{
		prevY = storepY[f-1];
		currY = storepY[f];
		count[f-1] = 0;
		for (y = 0; y < heightY; y++)
		{
			for (x = 0; x < row_sizeY;)
			{
				count[f-1] += abs((int)currY[x] - (int)prevY[x]);
				x++;
				if (quality == 0 || quality == 1)
				{
					if (!(x%4)) x += 12;
				}
			}
			prevY += pitchY;
			currY += pitchY;
		}
		if (quality == 1 || quality == 3)
		{
			prevU = storepU[f-1];
			prevV = storepV[f-1];
			currU = storepU[f];
			currV = storepV[f];
			for (y = 0; y < heightUV; y++)
			{
				for (x = 0; x < row_sizeUV;)
				{
					count[f-1] += abs((int)currU[x] - (int)prevU[x]);
					count[f-1] += abs((int)currV[x] - (int)prevV[x]);
					x++;
					if (quality == 1)
					{
						if (!(x%4)) x += 12;
					}
				}
				prevU += pitchUV;
				currU += pitchUV;
				prevV += pitchUV;
				currV += pitchUV;
			}
		}
	}

	/* Find the frame with the lowest difference count but
	   don't use the artificial duplicate at frame 0. */
	if (frame == 0)
	{
		lowest = count[1];
		lowest_index = 1;
	}
	else
	{
		lowest = count[0];
		lowest_index = 0;
	}
	for (x = 1; x < 12; x++)
	{
		if (count[x] < lowest)
		{
			lowest = count[x];
			lowest_index = x;
		}
	}
	/* adding some restrictions to avoid frame loss */
	last_result = frame + lowest_index;
	last_chosen=last_result;
	switch(last_result%12)
	{
	case 0:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result+7;
		break;
	case 2:
		if (mod_==0 || mod_==5)
		last_result=last_result-2;
		break;
	case 3:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result-2;
		break;
	case 5:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result+2;
		break;
	case 7:
		if (mod_==0 || mod_==5)
		last_result=last_result-7;
		break;
	case 8:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result-7;
		break;
	case 10:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result-9;
		break;
	default:
		break;
	}

	*forced_chosen = last_result;
	*chosen = last_chosen;
}