/*
rePal plugin by Bach, based on Donald Graft's decimate source.
March, 2003
created rePal, used to perform 60->25fps conversion in a more sensitive way.
-theory: assume that a PAL movie was telecined to NTSC. This is done in such
	 silly way that each group of 5 frames from the source (PAL) will be used to 
	 create 12 fields for the target (interlaced NTSC), most of them blended. 
	 I.e: suppose the frames A, B, C, D and E from the source. The target 
	 will have the fields A, AB, B, BC, C, C, CD, D, DE, E, EF, EF. As you
	 can see, only 6 out of each group of 12 fields are not blended.
	 My algorithm will search for the C field (frame) and use it as an origin
	 from which the frames A, B, D and E have constant distances.
	 Using a (smart)bob to do field->frame conversion, followed by repal(),
	 you will discard the blended and the repeated frames only.
*/

#include <stdio.h>
#include <windows.h>
#include "internal.h"
#include "version.h"

#define MAX_BLOCKS 50

#define GETFRAME(g, fp) \
{ \
	int GETFRAMEf; \
	GETFRAMEf = (g); \
	if (GETFRAMEf < 0) GETFRAMEf = 0; \
	if (GETFRAMEf > num_frames_hi - 1) GETFRAMEf = num_frames_hi - 1; \
	(fp) = child->GetFrame(GETFRAMEf, env); \
}

/* Discard 7-in-12 implementation. */
class rePal : public GenericVideoFilter
{
	int num_frames_hi, quality;
	int last_request, last_result, last_chosen, mod_;
	unsigned int hints[13];
	bool hints_invalid;
	bool firsttime;
	int heightY, row_sizeY, pitchY;
	int heightUV, row_sizeUV, pitchUV;
	unsigned int *sum, div;
	bool show;
	
public:
	rePal::rePal(PClip _child, int _quality, bool _show, IScriptEnvironment* env) :
		        GenericVideoFilter(_child), quality(_quality), show(_show)
	{
		int count = 0;
				
		/* setting new frame rate and framecount. */
		if (!vi.IsYV12() && !vi.IsYUY2())
			env->ThrowError("rePal: input must be YV12 or YUY2");
		if (quality < 0 || quality > 3)
			env->ThrowError("rePal: quality out of range (0-3)");
		
		num_frames_hi = vi.num_frames;
		vi.num_frames = vi.num_frames * 5 /12;
		vi.SetFPS(vi.fps_numerator *5, vi.fps_denominator * 12);
		
		last_request = -1;
		firsttime = true;
		sum = (unsigned int *) malloc(MAX_BLOCKS * MAX_BLOCKS * sizeof(unsigned int));
		if (sum == NULL)
			env->ThrowError("rePal: cannot allocate needed memory");
	}
	rePal::~rePal(void)
	{
		if (sum != NULL) free(sum);		
	}

    PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	PVideoFrame __stdcall rePal::GetFrameYV12(int inframe, IScriptEnvironment* env);
	PVideoFrame __stdcall rePal::GetFrameYUY2(int inframe, IScriptEnvironment* env);
	void __stdcall rePal::DrawShow(PVideoFrame &src, int useframe, int dropframe,
		                              int inframe, int calc_dropframe, IScriptEnvironment* env);
	void __stdcall rePal::DrawShowYUY2(PVideoFrame &src, int useframe, int dropframe,
		                              int inframe, int calc_dropframe, IScriptEnvironment* env);
    void __stdcall FindAnchor(int frame, int *forced_chosen, int *chosen, IScriptEnvironment* env);
    void __stdcall FindAnchorYUY2(int frame, int *forced_chosen, int *chosen, IScriptEnvironment* env);
};

