/*
rePal plugin by Bach, based on Donald Graft's decimate source.
March, 2003
created rePal, used to perform 60->25fps conversion in a more sensitive way.
-theory: assume that a PAL movie was telecined to NTSC. This is done in such
	 silly way that each group of 5 frames from the source (PAL) will be used to 
	 create 12 fields for the target (interlaced NTSC), most of them blended. 
	 I.e: suppose the frames A, B, C, D and E from the source. The target 
	 will have the fields A, AB, B, BC, C, C, CD, D, DE, E, EF, EF. As you
	 can see, only 6 out of each group of 12 fields are not blended.
	 My algorithm will search for the C field (frame) and use it as an origin
	 from which the frames A, B, D and E have constant distances.
	 Using a (smart)bob to do field->frame conversion, followed by repal(),
	 you will discard the blended and the repeated frames only.
*/

#include <stdio.h>
#include <windows.h>
#include "internal.h"
#include "version.h"
#include "utilities.h"
#include "strYUY2.h"
#include "repal.h"

void DrawStringYUY2(PVideoFrame &dst, int x, int y, const char *s);
 
void rePal::DrawShowYUY2(PVideoFrame &src, int useframe, int dropframe,
						       int inframe, int calc_dropframe, IScriptEnvironment* env)
{
	char buf[80];

	if (show == true)
	{
#ifdef MAKEWRITABLEHACK
		MakeWritable2(&src, env, vi);
#else
		env->MakeWritable(&src);
#endif
		sprintf(buf, "rePal %s", VERSION);
		DrawStringYUY2(src, 0, 0, buf);
		sprintf(buf, "Copyright 2003 Bach");
		DrawStringYUY2(src, 0, 1, buf);
		sprintf(buf,"in frm %d, use frm %d", inframe, useframe);
		DrawStringYUY2(src, 0, 3, buf);
		sprintf(buf,"anchor frm %d (type= %d)", dropframe, dropframe%12);
		DrawStringYUY2(src, 0, 4, buf);	
		sprintf(buf,"Should be frm %d (type= %d)", calc_dropframe, calc_dropframe%12);
		DrawStringYUY2(src, 0, 5, buf);	
		sprintf(buf,"difference %d", (dropframe-calc_dropframe)%12);
		DrawStringYUY2(src, 0, 6, buf);	
	}
}

PVideoFrame __stdcall rePal::GetFrameYUY2(int inframe, IScriptEnvironment* env)
{
	int dropframe, useframe, calc_dropframe;
	PVideoFrame src, next, dst;
	char buf[255];

		/* 60fps -> 25fps, 7 in 12 decimation*/
		/* Determine the correct set of frames to use and get them */
		useframe = inframe + inframe*7/5 ;
		FindAnchorYUY2((useframe/12)*12, &dropframe, &calc_dropframe, env);
		switch(dropframe%12)
		{
		case 1:
			useframe++;
			break;
		case 2:
			useframe+=2;
			break;
		case 3:
			useframe++;
			if(dropframe==useframe+2 || dropframe==useframe-5) useframe--;
			break;
		case 4:
			useframe+=2;
			if(dropframe==useframe+2 || dropframe==useframe-5) useframe--;
			break;		
		case 5:
			if(dropframe==useframe+1) useframe++;
			break;
		case 6:
			useframe++;
			if(dropframe==useframe+1) useframe++;
			break;
		case 7:
			useframe+=2;
			if(dropframe==useframe+1) useframe++;
			break;
		case 8:
			useframe++;
			if(dropframe==useframe+7) useframe--;
			break;
        case 9:
			useframe+=2;
			if(dropframe==useframe+7) useframe--;
			break;
		case 10:
			if(dropframe==useframe+6 || dropframe==useframe+1) useframe++;
			break;
		case 11:
			useframe++;
			if(dropframe==useframe+6 || dropframe==useframe+1) useframe++;
			break;
		default:
			break;
		}
	    
	   	GETFRAME(useframe, src);

		if (show == true)
		{
#ifdef MAKEWRITABLEHACK
			MakeWritable2(&src, env, vi);
#else
			env->MakeWritable(&src);
#endif
			env->MakeWritable(&src);
				sprintf(buf, "rePal %s", VERSION);
				DrawStringYUY2(src, 0, 0, buf);
				sprintf(buf, "Copyright 2003 Bach");
				DrawStringYUY2(src, 0, 1, buf);
				sprintf(buf,"in frm %d, use frm %d", inframe, useframe);
				DrawStringYUY2(src, 0, 3, buf);
				sprintf(buf,"anchor frm %d (type= %d)", dropframe, dropframe%12);
				DrawStringYUY2(src, 0, 4, buf);	
				sprintf(buf,"Should be frm %d (type= %d)", calc_dropframe, calc_dropframe%12);
				DrawStringYUY2(src, 0, 5, buf);	
				sprintf(buf,"difference %d", (dropframe-calc_dropframe)%12);
				DrawStringYUY2(src, 0, 6, buf);	
		}
	return src;		
}

void __stdcall rePal::FindAnchorYUY2(int frame, int *forced_chosen, int *chosen, IScriptEnvironment* env)

{
	int f;
	PVideoFrame store[13];
	const unsigned char *storep[13];
	const unsigned char *prev, *curr;
	int pitch;
	int row_size;
	int height;
	int x, y, lowest_index, div;
	unsigned int count[13], lowest;
	

	/* Only recalculate differences when a new set is needed. */
	if (frame == last_request)
	{
		*forced_chosen = last_result;
		*chosen = last_chosen;
		return;
	}
	last_request = frame;
	if (firsttime == true || frame == 0)
	{
		firsttime=false;
		last_result=1;
	}
	mod_=last_result%12;

	/* Get 13 frames starting at the one before the asked-for one. */
	for (f = 0; f <= 12; f++)
	{
		GETFRAME(frame + f - 1, store[f]);
		storep[f] = store[f]->GetReadPtr();
		hints_invalid = GetHintingData((unsigned char *) storep[f], &hints[f]);
	}

    pitch = store[0]->GetPitch();
    row_size = store[0]->GetRowSize();
    height = store[0]->GetHeight();

	/* Compare each frame to its predecessor. */
	for (f = 1; f <= 12; f++)
	{
		prev = storep[f-1];
		curr = storep[f];
		count[f-1] = 0;
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < row_size;)
			{
				count[f-1] += abs((int)curr[x] - (int)prev[x]);
				switch (quality)
				{
				case 0:
					x += 2;
					if (!(x%8)) x += 24;
					div = height * row_size / 8;
					break;
				case 1:
					x++;
					if (!(x%8)) x += 24;
					div = height * row_size / 8;
					break;
				case 2:
					x += 2;
					div = height * row_size / 2;
					break;
				case 3:
					x++;
					div = height * row_size / 2;
					break;
				}
			}
			prev += pitch;
			curr += pitch;
		}
	}

	/* Find the frame with the lowest difference count but
	   don't use the artificial duplicate at frame 0. */
	if (frame == 0)
	{
		lowest = count[1];
		lowest_index = 1;
	}
	else
	{
		lowest = count[0];
		lowest_index = 0;
	}
	for (x = 1; x < 12; x++)
	{
		if (count[x] < lowest)
		{
			lowest = count[x];
			lowest_index = x;
		}
	}
	/* adding some restrictions to avoid frame loss */
	last_result = frame + lowest_index;
	last_chosen=last_result;
	switch(last_result%12)
	{
	case 0:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result+7;
		break;
	case 2:
		if (mod_==0 || mod_==5)
		last_result=last_result-2;
		break;
	case 3:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result-2;
		break;
	case 5:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result+2;
		break;
	case 7:
		if (mod_==0 || mod_==5)
		last_result=last_result-7;
		break;
	case 8:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result-7;
		break;
	case 10:
		if (mod_==2 || mod_==4 || mod_==7 || mod_==9 || mod_==11)
		last_result=last_result-9;
		break;
	default:
		break;
	}
	
	*forced_chosen = last_result;
	*chosen = last_chosen;
}

AVSValue __cdecl Create_rePal(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	char path[1024];
	char buf[80], *p;
	int quality = 2;
	bool show = false;
	
	try
	{
		FILE *f;

		const char* plugin_dir = env->GetVar("$PluginDir$").AsString();
		strcpy(path, plugin_dir);
		strcat(path, "\\rePal.def");
		if ((f = fopen(path, "r")) != NULL)
		{
			while(fgets(buf, 80, f) != 0)
			{
				if (strncmp(buf, "quality=", 8) == 0)
				{
					p = buf;
					while(*p++ != '=');
					quality = atoi(p);
				}
				
				if (strncmp(buf, "show=", 5) == 0)
				{
					p = buf;
					while(*p++ != '=');
					if (*p == 't') show = true;
					else show = false;
				}
			}
			fclose(f);
		}
	}
	catch (...)
	{
		// plugin directory not set
		// probably using an older version avisynth
	}

    return new rePal(args[0].AsClip(),			// clip
						args[1].AsInt(quality),		// quality level
						args[2].AsBool(show),		// show
						env);
}
